import QtQuick 1.1
import se.flir 1.0

// Archive
// -------
// The archive. Uses a ThumbnailArchive and a PreviewArchive to show the archive images.

Item{
    // Signals and properties
    signal hideArchive
    // signal showThumbnailArchive
    // signal showPreviewArchive

    // signals that the C++ model connects too.
    signal thumbnailDelegateDestroyed(int index)
    signal previewDelegateDestroyed(int index)


    // Memory for the archive state.
    property variant lastArchiveView: ArchiveHandler.Thumbnails

    id: root
    width: grid.width
    height: grid.height

    Rectangle {
        id: archiveBackground
        anchors.fill: parent
        color: colors.background
    }

    // Debug output
//    onHideArchive: console.log("Archive::hideArchive()")
//    onThumbnailDelegateDestroyed: console.log("Archive::thumbnailDelegateDestroyed(" + index + ")")
//    onPreviewDelegateDestroyed: console.log("Archive::previewDelegateDestroyed(" + index + ")")

    // Handle big states
    function hide() {
//        console.log("Hiding archive")
        lastArchiveView = archiveHandler.view
        archiveHandler.view = ArchiveHandler.Invisible
    }

    function show() {
//        console.log("Showing archive")
        scope.forceActiveFocus()
        archiveHandler.view = lastArchiveView
    }

    FocusScope {
        anchors.fill: parent
        id: scope

        // Thumbnail archive
        ThumbnailArchive {
            id: thumbnailArchive
            onHideArchive: {
                root.hideArchive()
            }
            onShowPreviewArchive: archiveHandler.view = ArchiveHandler.ImageSelector
            onDelegateDestroyed: root.thumbnailDelegateDestroyed(index)
            focus: root.state == "thumbnailView"
        }

        // Preview archive
        PreviewArchive {
            id: previewArchive
            onShowThumbnailArchive: archiveHandler.view = ArchiveHandler.Thumbnails
            onDelegateDestroyed: root.previewDelegateDestroyed(index)
            onShowArchiveToolBar: menus.navigateUp()
            focus: root.state == "imageSelectorView"
        }

        // Video playback overlay
        VideoPlaybackOverlay {
            id: videoPlaybackOverlay
            focus: root.state == "fullscreenVideo"
        }

        // No images message
        MessageBox {
            id: emptyArchiveNotification
            //: Message header. Displayed when user enters the image archive and there are no saved images on the memory card.
            //% "No images found"
            title: qsTrId("ID_ARCHIVE_NO_IMAGES_HEADER") + translation.update
            //: Message text body. Displayed with message header when no images are found.
            //% "There are no saved images."
            text: qsTrId("ID_ARCHIVE_NO_IMAGES_BODY") + translation.update
            hasButtons: false
            iconStyle: iconEnum.info
            focus: root.state == "archiveIsEmpty"

            // Keyboard handling
            Keys.onReleased: {
                if (event.key === keys.archive || event.key === keys.back) {
                    hideArchive()
                    event.accepted = true
                }
            }
        }

        // Indexing images message
        MessageBox {
            id: indexingNotification
            //: Message header. Displayed when user enters the image archive and the archive is indexing the data.
            //% "Indexing images"
            title: qsTrId("ID_ARCHIVE_INDEXING_IMAGES_HEADER") + translation.update
            //: Message text body. Displayed with message header when the archive is indexing the data.
            //% "Reading image data..."
            text: qsTrId("ID_ARCHIVE_INDEXING_IMAGES_BODY") + translation.update
            hasButtons: false
            iconStyle: iconEnum.info
            focus: root.state == "archiveIndexing"

            // Keyboard handling
            Keys.onReleased: {
                if (event.key === keys.archive || event.key === keys.back) {
                    hideArchive()
                    event.accepted = true
                }
            }
        }

        Image {
            id: fullscreenImage
            anchors.fill: parent
            sourceSize.width: grid.width
            sourceSize.height: grid.height
            width: grid.width
            height: grid.height
            cache: false
            smooth: true
            asynchronous: true
            focus: root.state == "fullscreen"

            Behavior on opacity {
                NumberAnimation {
                    duration: 300
                    easing.type: Easing.InOutQuad
                }
            }

            Keys.onReleased: {
				// Check archive button too, to avoid focus loss when leaving & returning to archive
                if (event.key === keys.back || event.key === keys.archive) {
                    archiveHandler.view = ArchiveHandler.ImageSelector
                    event.accepted = true
                }
                else if(event.key === keys.select) {
                    menus.navigateUp()
                    event.accepted = true
                }
            }
        }

        MessageBox {
            id: deleteImageQuestion

            //: Question. Displayed when user selects to delete an image file. %1 is a placeholder for a filename.
            //% "Delete %1?"
            title: qsTrId("DELETE_X_HEADER").arg(archiveModel.currentGroupName)
            //: Information text displayed with question when user selects to delete an image or video file.
            //% "This will delete the current image."
            text: qsTrId("DELETE_X_BODY") +  translation.update
            hasButtons: true
            iconStyle: iconEnum.warning
            //: Imperative. Button. Select to delete image
            //% "Delete"
            okButtontext: qsTrId("ID_BUTTON_DELETE") +  translation.update
            //: Imperative. Button. Select to cancel delete action
            //% "Cancel"
            cancelButtontext: qsTrId("ID_BUTTON_CANCEL") +  translation.update
            focus: root.state == "deleteImageDialog"

            onCancelSelected: {
                archiveHandler.view = ArchiveHandler.ImageSelector
                previewArchive.forceActiveFocus()
            }
            onOkSelected: {
                archiveHandler.removeCurrentImage();
                archiveHandler.view = ArchiveHandler.ImageSelector
                // Really convince qml about the focus for empty archives...
                if (archiveModel.empty)
                    emptyArchiveNotification.forceActiveFocus()
                else
                    previewArchive.forceActiveFocus()
            }
        }

        // Message box
        MessageBox {
            id: archiveInfoDiag
            focus: root.state == "infoDialog"
            title: archiveModel.currentGroupFullName
            tableModel: infoMsgBoxModel
            onCancelSelected: {
                archiveHandler.view = ArchiveHandler.ImageSelector
                previewArchive.forceActiveFocus()
            }
        }


        // Keyboard handling
        Keys.onPressed: {
            if (event.key === keys.archive) {
                // Prevent this from falling through to desktop
                event.accepted = true
            }
        }
        Keys.onReleased: {
            if (event.key === keys.archive || event.key === keys.back) {
                hideArchive()
                event.accepted = true
            }
        }
    }

    ListModel {     // This model is populated when opening info dialog
        id: infoMsgBoxModel
    }

    Connections { target: menus.menuItem("archive_delete")
        onTriggered: {
            archiveHandler.view = ArchiveHandler.Fullscreen;
            root.state = "deleteImageDialog"
        }
    }

    // Listen to changes in the model and set the corresponding view.
    // Note: this generates an error at startup, as not everything is setup, but it works!
    Connections { target: archiveModel
        onCurrentIndexChanged: {
            if (state == "thumbnailView") {
                if (thumbnailArchive.currentIndex !== archiveModel.currentIndex) {
                    thumbnailArchive.positionViewAtIndex(archiveModel.currentIndex, ListView.Contain)
                    thumbnailArchive.currentIndex = archiveModel.currentIndex
                }
            } else if (state == "imageSelectorView") {
                if (previewArchive.currentIndex !== archiveModel.currentIndex) {
                    previewArchive.positionViewAtIndex(archiveModel.currentIndex, ListView.Contain)
                    previewArchive.currentIndex = archiveModel.currentIndex
                }
            }
        }
        // Invalidate the stored index when re-reading stuff
        onStatusChanged: {
            if (archiveModel.status == ArchiveModel.StatusStarting) {
                thumbnailArchive.currentIndex = -1
            }
        }
    }

    Connections { target: menus.menuItem("archive_info")
        onTriggered: {
            archiveInfoDiag.hasButtons = false
            archiveInfoDiag.iconStyle = archiveInfoDiag.iconEnum.info
            archiveHandler.view = ArchiveHandler.Fullscreen;
            root.state = "infoDialog"
        }
    }

    Connections {
        target: archiveModel
        onCurrentArchiveDataChanged: {
            infoMsgBoxModel.clear()
            if (archiveModel.currentGroupIRWidth !== 0)
            {
                //: Noun. Emissivity of image.
                //% "Emissivity:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_EMISSIVITY"),
                        // 0x3b5 is the unicode code point for epsilon
                        "value": "\u03b5 " + archiveModel.currentGroupEmissivity})
                //: Noun. Reflected temperature of image.
                //% "Reflected temperature:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_REFLECTED_TEMPERATURE"), "value": archiveModel.currentGroupReflTemp + qsTrId(greenbox.resultTable.tempUnit)})
                //: Noun. Resoultion of IR image
                //% "Thermal resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_IR_RESOLUTION"),
                                          "value": archiveModel.currentGroupIRWidth + "x" + archiveModel.currentGroupIRHeight})
                //: Noun. Resolution of visual image.
                //% "Digital resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_VIS_RESOLUTION"),
                                         "value": archiveModel.currentGroupDCWidth + "x" + archiveModel.currentGroupDCHeight})
                //: Noun. Lens (field of view of lens used to take image)
                //% "Lens:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_LENS"),
                                          "value": archiveModel.currentGroupFOV + "\u00B0"}) // 0xB0 is the unicode point for degrees.
            }
            else if (archiveModel.currentGroupIsVideo)
            {
                //: Noun. Resoultion of video
                //% "Video resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_VIDEO_RESOLUTION"),
                                          "value": archiveModel.currentGroupImgWidth + "x" + archiveModel.currentGroupImgHeight})
            }
            else if (archiveModel.currentGroupImgWidth !== 0)
            {
                //: Noun. Resolution of visual image.
                //% "Digital resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_VIS_RESOLUTION"),
                                         "value": archiveModel.currentGroupImgWidth + "x" + archiveModel.currentGroupImgHeight})
            }
            else if (archiveModel.currentGroupIsVideo)
            {
                //: Noun. Resoultion of video
                //% "Video resolution:"
                infoMsgBoxModel.append({ "label": qsTrId("ID_ARCHIVE_VIDEO_RESOLUTION"),
                                          "value": archiveModel.currentGroupImgWidth + "x" + archiveModel.currentGroupImgHeight})
            }
            else
            {
                //: While waiting for the information to be fetched.
                //% "One moment"
                infoMsgBoxModel.append({ "label": qsTrId("ID_WAITFORIT"), "value": ""})
            }
        }
    }

    Connections { // get keyboard focus from menu to the right thing after it closes.
        target: menus
        onMenuClosed: {
            if (state == "fullscreen") {
                fullscreenImage.forceActiveFocus()
            } else if (state == "imageSelectorView") {
                previewArchive.forceActiveFocus()
            }
            else if (state == "fullscreenVideo") {
                videoPlaybackOverlay.forceActiveFocus()
            }
        }
    }

    states: [
        State {
            name: "hidden"
            when: archiveHandler.view == ArchiveHandler.Invisible
            PropertyChanges { target: root; visible: false }
        },
        State {
            name: "visible"
            PropertyChanges { target: root; visible: true }
            PropertyChanges { target: archiveBackground; visible: true }
            PropertyChanges { target: thumbnailArchive; opacity: 0 }
            PropertyChanges { target: previewArchive; visible: false }
            PropertyChanges { target: archiveInfoDiag; opacity: 0 }
            PropertyChanges { target: deleteImageQuestion; opacity: 0 }
            PropertyChanges { target: fullscreenImage; opacity: 0 }
            PropertyChanges { target: emptyArchiveNotification; opacity: 0 }
            PropertyChanges { target: fullscreenImage; source: "" }
            PropertyChanges { target: videoPlaybackOverlay; visible: false }
        },
        State {
            name: "archiveIsEmpty"
            extend: "visible"
            when: archiveHandler.view != ArchiveHandler.Invisible && archiveModel.empty && archiveModel.status != ArchiveModel.StatusIndexing
            PropertyChanges { target: emptyArchiveNotification; opacity: 1 }
            StateChangeScript { script: console.log("state = archiveIsEmpty") }
        },
        State {
            name: "archiveIsIndexing"
            extend: "visible"
            when: archiveHandler.view != ArchiveHandler.Invisible && archiveModel.empty && archiveModel.status == ArchiveModel.StatusIndexing
            PropertyChanges { target: indexingNotification; opacity: 1 }
            StateChangeScript { script: console.log("state = archiveIsIndexing") }
        },
        State {
            name: "thumbnailView"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.Thumbnails && archiveModel.status === ArchiveModel.StatusReady
            PropertyChanges { target: thumbnailArchive; opacity: 1 }
            StateChangeScript {
                script: {
                    console.log("state = thumbnailView")
                    if (thumbnailArchive.currentIndex !== archiveModel.currentIndex) {
                        thumbnailArchive.positionViewAtIndex(archiveModel.currentIndex, ListView.Contain)
                        thumbnailArchive.currentIndex = archiveModel.currentIndex
                    }
                }
            }
        },
        State {
            name: "imageSelectorView"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.ImageSelector && archiveModel.status === ArchiveModel.StatusReady
            PropertyChanges { target: previewArchive; visible: true }
            StateChangeScript {
                script: {
                    console.log("state = imageSelectorView")
                    if (previewArchive.currentIndex !== archiveModel.currentIndex) {
                        previewArchive.positionViewAtIndex(archiveModel.currentIndex, ListView.Contain)
                        previewArchive.currentIndex = archiveModel.currentIndex
                    }
                }
            }
        },
        State {
            name: "fullscreen"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.Fullscreen
            PropertyChanges { target: fullscreenImage; opacity: 1 }
            PropertyChanges { target: fullscreenImage; source: previewArchive.currentItem.frontSource }
            StateChangeScript { script: console.log("fullscreen state got started") }
        },

        State {
            name: "fullscreenVideo"
            extend: "visible"
            when: archiveHandler.view == ArchiveHandler.FullscreenVideo
            PropertyChanges { target: archiveBackground; visible: false }
            PropertyChanges { target: videoPlaybackOverlay; visible: true }
            StateChangeScript { script: videoPlaybackOverlay.forceActiveFocus() }
            StateChangeScript { script: console.log("fullscreenvideo state got started") }
        },
        State {
            name: "infoDialog"
            extend: "imageSelectorView"
            when: archiveHandler.view == ArchiveHandler.Fullscreen
            PropertyChanges { target: archiveInfoDiag; opacity: 1 }
            StateChangeScript { script: console.log("state = infoDialog") }
            StateChangeScript { script: archiveInfoDiag.show() }
        },
        State {
            name: "deleteImageDialog"
            extend: "imageSelectorView"
            PropertyChanges { target: deleteImageQuestion; opacity: 1 }
            StateChangeScript { script: deleteImageQuestion.show() }
        }
    ]
}
