import QtQuick 1.1

// MessageBox
// ----------
// A modal message box, with an OK and a CANCEL button.

Item {

    // Own properties
    property alias title: titleItem.text
    property alias text: textBoxItem.text
    property alias cancelButtontext: cancelButton.text
    property alias okButtontext: okButton.text
    property alias tableModel: textTableItem.model
    property bool hasButtons: true
    property bool acceptKeyRelease : false
    property bool tableMode: (tableModel === null || tableModel === undefined) ? false : true

    // Property values
    width: grid.width
    height: grid.height
    opacity: 0
    state: "small"

    // Signals
    signal okSelected
    signal cancelSelected

    // Pseudo icon enum
    property string iconStyle: iconEnum.warning
    property alias iconEnum: iconEnum
    QtObject {
        id: iconEnum
        property string warning: "../images/Ic_GlobalDialogue_Warning_Sel.png"
        property string info: "../images/Ic_GlobalDialogue_Info_Sel.png"
        property string question: "../images/Ic_GlobalDialogue_Questionmark_Sel.png"
        property string error: "../images/Ic_GlobalDialogue_Error_Sel.png"
    }

    // Big or small state
    states: [
        State {
            name: "small"
            PropertyChanges { target: canvas; width: grid.messageBoxWidth }
            PropertyChanges { target: canvas; height: grid.messageBoxHeight }
            PropertyChanges { target: canvas; height: grid.messageBoxHeight }
            PropertyChanges { target: okButton; state: "small" }
            PropertyChanges { target: cancelButton; state: "small" }
        },
        State {
            name: "big"
            PropertyChanges { target: canvas; width: grid.messageBoxBigWidth }
            PropertyChanges { target: canvas; height: grid.messageBoxBigHeight }
            PropertyChanges { target: okButton; state: "big" }
            PropertyChanges { target: cancelButton; state: "big" }
        }
    ]

    onTableModeChanged: {
        if (tableMode || titleItem.lineCount > 2 || textBoxItem.lineCount > 3)
            state = "big"
    }
    onTextChanged: {
        if (tableMode || titleItem.lineCount > 2 || textBoxItem.lineCount > 3)
            state = "big"
    }
    onTitleChanged: {
        if (tableMode || titleItem.lineCount > 2 || textBoxItem.lineCount > 3)
            state = "big"
    }

    // Debug output
    //onOkSelected: console.log("MessageBox::okSelected()")
    //onCancelSelected: console.log("MessageBox::cancelSelected()")
    //onStateChanged: console.log("MessageBox: State = " + state)

    // Show and hide functions

    function show() {
        //console.log("MessageBox::show()")
        cancelButton.selected = true
        opacity = 1
        forceActiveFocus()
    }

    function hide() {
        //console.log("MessageBox::hide()")
        opacity = 0
        focus = false
        acceptKeyRelease = false
    }

    // Touch handling

    MouseArea {
        anchors.fill: parent
        onClicked: {
            if (hasButtons === false)
                cancelSelected()
        }
    }

    // The message box

    Image {
        id: messageBox
        anchors.fill: parent
        source: parent.state === "big" ? "../images/Bg_DialoguePopUp_BigBg_Def.png" : "../images/Bg_DialoguePopUp_SmallBg_Def.png"

        Rectangle {
            id: canvas
            color: "transparent"
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter

            // The icon

            Image {
                id: icon
                anchors.verticalCenter: titleItem.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing * 2
                source: iconStyle
            }

            // The title text

            Text {
                id: titleItem
                anchors.top: parent.top
                anchors.topMargin: grid.verticalSpacing * 3
                anchors.left: icon.right
                anchors.leftMargin: grid.horizontalSpacing * 2
                width: parent.width - icon.width - grid.horizontalSpacing * 4
                font.family: fonts.family
                font.pixelSize: fonts.smallSize
                font.bold: true
                color: colors.textFocused
                wrapMode: Text.Wrap
                maximumLineCount: 3
            }

            // The blue line

            BorderImage {
                id: blueLine
                anchors.top: titleItem.bottom
                anchors.topMargin: titleItem.lineCount === 1 ? grid.verticalSpacing * 3 : grid.verticalSpacing * 2
                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing
                anchors.right: parent.right
                anchors.rightMargin: grid.horizontalSpacing
                border { left: 50; top: 0; right: 50; bottom: 0 }
                source: "../images/Bg_DialoguePopUp_Line_Def.png"
            }

            // The message

            Item {
                id: textItem
                //property alias text: textBoxItem.text
                anchors.top: blueLine.bottom
                anchors.topMargin: grid.verticalSpacing * 3
                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing * 5
                anchors.right: parent.right
                anchors.rightMargin: grid.horizontalSpacing * 5
                anchors.bottom: parent.bottom
                anchors.bottomMargin: grid.verticalSpacing * 3

                // Show text in text box
                Text {
                    id: textBoxItem
                    visible: tableMode === false
                    anchors.fill: parent
                    maximumLineCount: 6
                    wrapMode: Text.WordWrap
                    font.family: fonts.family
                    font.pixelSize: fonts.smallSize
                    color: colors.textFocused
                }

                // or: Show text from model in a table (see archive info)
                ListView {
                    id: textTableItem
                    visible: tableMode
                    anchors.fill: parent
                    spacing: grid.verticalSpacing

                    delegate: Item {
                        id: delegateId
                        width: parent.width
                        height: delegateLabel.height
                        Text {
                            id: delegateLabel
                            font.family: fonts.family
                            font.pixelSize: fonts.smallSize
                            color: colors.textFocused
                            anchors.left: parent.left
                            text: label
                        }
                        Text {
                            id: delegateValue
                            font.family: fonts.family
                            font.pixelSize: fonts.smallSize
                            color: colors.textFocused
                            anchors.right: parent.right
                            text: value
                        }
                    }
                }
            }

            // The buttons

            Button {
                id: okButton
                anchors.left: parent.left
                anchors.leftMargin: grid.horizontalSpacing
                anchors.bottom: parent.bottom
                anchors.bottomMargin: grid.verticalSpacing
                text: "YES"
                visible: hasButtons

                onSelectedChanged: if (selected) cancelButton.selected = false
                onClicked: okSelected()
            }

            Button {
                id: cancelButton
                anchors.right: parent.right
                anchors.rightMargin: grid.horizontalSpacing
                anchors.bottom: parent.bottom
                anchors.bottomMargin: grid.verticalSpacing
                text: "NO"
                visible: hasButtons

                selected: true
                onSelectedChanged: if (selected) okButton.selected = false
                onClicked: cancelSelected()
            }
        }
    }

    // Keyboard handling
    Keys.onPressed: {
        acceptKeyRelease = true
        if (event.key === keys.up || event.key === keys.down ||
            event.key === keys.left || event.key === keys.right ||
            event.key === keys.back)
        {
            // Prevent leaking keys to parent
            event.accepted = true
        }
    }

    Keys.onReleased: {
        if (acceptKeyRelease)
        {
            if (hasButtons === false)
            {
                cancelSelected()
                event.accepted = true
            }
            else
            {
                if (event.key === keys.left) {
                    if (cancelButton.selected)
                        okButton.selected = true
                    event.accepted = true
                }
                else if (event.key === keys.right) {
                    if (okButton.selected)
                        cancelButton.selected = true
                    event.accepted = true
                }
                else if (event.key === keys.select) {
                    if (okButton.selected)
                        okSelected()
                    else
                        cancelSelected()
                    event.accepted = true
                }
                else if (event.key === keys.back || event.key === keys.archive) {
                    cancelSelected()
                    event.accepted = true
                }
                else if (event.key === keys.up || event.key === keys.down) {
                    // Prevent leaking keys to parent
                    event.accepted = true
                }
            }
        }
    }
}
