import QtQuick 1.1

// ResultTableDelegate
// -------------------
// Delegate for the ResultTable element.
// the result table has measurement items (functions), this delegate will display the details for one of those.
// we show the name, the temparature and more rows for more complex types.

Item {

    // Property values
    id: resultTableDelegate
    height: column.height + 2

    property int column1Width: 0
    property int column2Width: 0

    signal columnWidthsChanged

    // The background
    RoundedRect {
        anchors.fill: parent
        anchors.bottomMargin: 2
    }

    Column {
        id: column
        x: grid.horizontalSpacing
        width: parent.width

        function recalcWidth() {
            var c1, c2;
            if (row1.column2Width == 0) { // a label, not a name-value pair
                c1 = row1.column1Width / 2
                c2 = row1.column2Width / 2
            } else {
                c1 = row1.column1Width + grid.horizontalSpacing
                c2 = row1.column2Width + grid.horizontalSpacing
            }

            c1 = Math.max(c1, row2.column1Width + grid.horizontalSpacing);
            c2 = Math.max(c2, row2.column2Width + grid.horizontalSpacing);
            c1 = Math.max(c1, row3.column1Width + grid.horizontalSpacing);
            c2 = Math.max(c2, row3.column2Width + grid.horizontalSpacing);
            c1 = Math.max(c1, row4.column1Width + grid.horizontalSpacing);
            c2 = Math.max(c2, row4.column2Width + grid.horizontalSpacing);
            c1 = Math.max(c1, row5.column1Width + grid.horizontalSpacing);
            c2 = Math.max(c2, row5.column2Width + grid.horizontalSpacing);

            var changed = false
            if (c1 > resultTableDelegate.column1Width || c1 < resultTableDelegate.column1Width - 5) {
                changed = true
                resultTableDelegate.column1Width = c1;
            }
            if (c2 > resultTableDelegate.column2Width || c2 < resultTableDelegate.column2Width - 5) {
                changed = true
                resultTableDelegate.column2Width = c2;
            }
            if (changed) {
                resultTableDelegate.columnWidthsChanged()
            }
        }

        ResultTableDelegateRow {
            id: row1
            _label: {
                if (type === "spot")
                {
                    if (typeCount > 1)
                    {
                        //: Label for spot meter result in measure result table. Keep as short as possible. '%1' is a placeholder for an ID (number).
                        //% "spot %1"
                        return qsTrId("ID_RESULT_SPOT_INDEX").arg(typeIndex + 1) + translation.update
                    }
                    else if (resultTable.totalCount > 1)
                    {
                        //: Label for spot meter result in measure result table. Keep as short as possible.
                        //% "spot"
                        qsTrId("ID_RESULT_SPOT") + translation.update
                    }
                    else
                        return ""
                }
                else if (type === "box")
                {
                    if (resultTable.totalCount === 1)
                        return ""
                    if (typeCount === 1)
                    {
                        //: Label for box meter result in measure result table. Keep as short as possible.
                        //% "box"
                        return qsTrId("ID_RESULT_BOX")
                    }
                    //: Label for box meter result in measure result table. Keep as short as possible. '%1' is a placeholder for an ID (number).
                    //% "box %1"
                    return qsTrId("ID_RESULT_BOX_INDEX").arg(typeIndex + 1) + translation.update
                }
                else if (type === "circle")
                {
                    if (typeCount === 1)
                        return ""
                    else
                    {
                        //: Label for circle meter result in measure result table. Keep as short as possible. '%1' is a placeholder for an ID (number).
                        //% "circle %1"
                        return qsTrId("ID_RESULT_CIRCLE_INDEX").arg(typeIndex + 1) + translation.update
                    }
                }
                else if (type === "line")
                {
                    if (typeCount === 1)
                        return ""
                    else
                    {
                        //: Label for line meter result in measure result table. Keep as short as possible. '%1' is a placeholder for an ID (number).
                        //% "line %1"
                        return qsTrId("ID_RESULT_LINE_INDEX").arg(typeIndex + 1) + translation.update
                    }
                }
                else if (type === "refTemp")
                {
                    //: Label for a reference temperature in measure result table. Keep as short as possible.
                    //% "temp"
                    return qsTrId("ID_RESULT_REF_TEMP") + translation.update
                }
                else if (type === "diff")
                {
                    if (preset === "deltaSpot")
                    {
                        //: Label for a delta function in measure result table. 'D' should be replaced by Greek captital delta. 's' should be the first letter of 'spot'.
                        //% "D s"
                        return qsTrId("ID_RESULT_DELTA_SPOT") + translation.update
                    }
                    else if (preset === "deltaTemp")
                    {
                        //: Label for a delta function in measure result table. 'D' should be replaced by Greek captital delta. 't' should be the first letter of 'temperature'.
                        //% "D t"
                        return qsTrId("ID_RESULT_DELTA_TEMP") + translation.update
                    }
                    else
                    {
                        return qsTrId(diff1).arg(diffId1) + "-" + qsTrId(diff2).arg(diffId2)
                    }
                }
                else if (type === "meterlink")
                {
                    return meterlinkUnit
                }

                else
                    return ""
            }
            _value: (type === "diff" && preset === "") ? "" : value
            onWidthsChanged: parent.recalcWidth();
        }

        ResultTableDelegateRow {
            id: row2
            //: Abbreviation of 'maximum'
            //% "max"
            _label: maxValue === "" ? "" : (typeCount === 1 ? "" : "  ") + qsTrId("ID_RESULT_MAX") + translation.update
            _value: maxValue
            labelType: "maxLabel"
            valueType: "maxValue"
            onWidthsChanged: parent.recalcWidth();
        }

        ResultTableDelegateRow {
            id: row3
            //: Abbreviation of 'minimum'
            //% "min"
            _label: minValue === "" ? "" : (typeCount === 1 ? "" : "  ") + qsTrId("ID_RESULT_MIN") + translation.update
            _value: minValue
            labelType: "minLabel"
            valueType: "minValue"
            onWidthsChanged: parent.recalcWidth();
        }

        ResultTableDelegateRow {
            id: row4
            //: Abbreviation of 'average'
            //% "avg"
            _label: avgValue === "" ? "" : (typeCount === 1 ? "" : "  ") + qsTrId("ID_RESULT_AVG") + translation.update
            _value: avgValue
            labelType: "avgLabel"
            valueType: "avgValue"
            onWidthsChanged: parent.recalcWidth();
        }

        // Used for delta function in advanced mode only
        ResultTableDelegateRow {
            id: row5
            _label: (type === "diff" && preset === "") ? "\u0394" : "" // 0x394 is the unicode codepoint for Greek captital letter delta
            _value: (type === "diff" && preset === "") ? value : ""
            labelType: "diffLabel"
            valueType: "diffValue"
            onWidthsChanged: parent.recalcWidth();
        }
    }
}
