import QtQuick 1.1

//
// Settings page - slides in from right in full screen
//

Item {
    property Item previous: null
    property Item next: null
    //: Noun. Header in 'options mode' main menu, meaning a group of settings.
    //% "Settings"
    property string name: qsTrId("ID_TOOLBAR_OPTIONS") + translation.update
    property alias model: settingsView.model
    property alias currentIndex: settingsView.currentIndex
    property bool acceptKeyRelease: false
    property alias basePage: settingsHeader.basePage
    property bool returnStateArchive: false

    signal hideSignal
    signal stateChangeRequestInvoked(string str)

    property Component internalDelegate: SettingsDelegate {
        onClicked: invokeCurrent("")
    }

    id: page
    x: grid.width
    y: 0
    width: grid.width
    height: grid.height
    opacity: 1

    // Animations used to slide pages in and out

    ParallelAnimation {
        id: showAnimation
        NumberAnimation {
            target: page; property: "x"; to: previous === null ? 0 : grid.width;
            duration: previous !== null ? 200 : 0; easing.type: Easing.InOutQuad
        }
        ScriptAction {
            script: {
                if (previous !== null) {
                    previous.runHideParentAnimation()
                    previous.model.isNowActive(false)
                }
            }
        }
    }

    NumberAnimation {
        id: hideParentAnimation
        target: page; property: "x"; to: previous === null ? -grid.width : 0;
        duration: 200; easing.type: Easing.InOutQuad
    }

    NumberAnimation {
        id: showParentAnimation
        target: page; property: "x"; to: previous === null ? 0 : grid.width;
        duration: 200; easing.type: Easing.InOutQuad
    }

    SequentialAnimation {
        id: hideAnimation
        ParallelAnimation {
            NumberAnimation {
                target: page; property: "x"; to: grid.width;
                duration: previous !== null ? 200 : 0; easing.type: Easing.InOutQuad
            }
            ScriptAction { script: { if (previous !== null) { previous.runShowParentAnimation();} } }
        }
        ScriptAction { script: { focus = false; if (previous !== null) {previous.forceActiveFocus(); } hideSignal() } }
    }

    // Functions needed to access animations in parent

    function runShowParentAnimation() { showParentAnimation.start() }
    function runHideParentAnimation() { hideParentAnimation.start() }

    // Functions used to show and hide pages

    function show ()
    {
        //console.log("Showing SettingsPage " + name)
        showAnimation.start()
        forceActiveFocus()
        if (model !== undefined && model.isNowActive !== undefined)
            model.isNowActive(true)
        returnStateArchive = false
    }

    function hide ()
    {
        //console.log("Hiding SettingsPage " + name)
        hideAnimation.start()
        if (returnStateArchive)
            stateChangeRequestInvoked("archiveState")
        else
            stateChangeRequestInvoked("defaultState")
        acceptKeyRelease = false
        if (model !== undefined && model.isNowActive !== undefined)
            model.isNowActive(false)
    }

    function hideAll()
    {
        if (previous === null)
            hide()
        else
            hideSignal()
        while(previous !== null)
        {
            previous.returnStateArchive = returnStateArchive
            if (previous.previous === null)
            {
                previous.forceActiveFocus()
                previous.hide()
            }
            else
            {
                previous.hideSignal()
            }
            previous = previous.previous
        }
    }

    function invokeCurrent (selectKey)
    {
        if (settingsView.currentItem.item === undefined)
            return

        // If item has next, load it
        if (settingsView.currentItem.item.delegateNextQML !== undefined && settingsView.currentItem.item.delegateNextQML !== "")
        {
            pageLoader.source = settingsView.currentItem.item.delegateNextQML
        }
        else if (selectKey !== keys.right)  // Only allow right key to navigate
        {
            if (settingsView.currentItem.item.delegateConfirmMsg !== "")
            {
                // Otherwise, if item has confirmation question, display it
                messageLoader.source = "MessageBox.qml"
            }
            else
            {
                // Otherwise, if item has setting, set it
                model.setProperty(settingsView.currentIndex, settingsView.currentItem.item.delegateID, settingsView.currentItem.item.delegateValue)

                if (basePage === false && settingsView.currentItem.item.delegateAutoHidePage) {
                    if (settingsView.currentItem.item.delegateSettingRadio) {
                        hideTimer.start()
                    }
                    else if ( ! settingsView.currentItem.item.delegateSettingCheckBox) {
                        hide()
                    }
                }
            }
        }
    }

    // Timers

    Timer {
        id: hideTimer
        interval: 300
        onTriggered: hide()
    }

    Timer {
        id: hiddenServiceMenuTimer
        interval: 10000
        onTriggered: {
            pageLoader.source = "SettingsPage.qml"
            greenbox.system.showHiddenMenu()    // Load any data in the menus. Somewhat hacky, but it is a special case (i.e. don't copy this idea!)
        }
    }
    // This timer is needed to stop the hiddenservicemenutimer if we have ever received any onPressed repeat events
    // as we then need to ignore onReleased events to avoid stepping deeper into the setup hierarchy.
    Timer {
        id: hiddenServiceWatchdogTimer
        interval: 1000
        onTriggered: {
            if (hiddenServiceMenuTimer.running)
                hiddenServiceMenuTimer.stop()
        }
    }

    Timer {
        id: escapeSettingsModeTimer
        interval: 1000
        onTriggered: hideAll()
    }

    // Key handling

    Keys.onPressed: {
        acceptKeyRelease = true
        event.accepted = true
        if (event.key === keys.right)
        {
            if (event.isAutoRepeat === false)
            {
                if (hiddenServiceMenuTimer.running === false
                        && settingsView.currentItem !== null
                        && settingsView.currentItem.item !== undefined
                        && settingsView.currentItem.item.delegateNextModel !== undefined)
                {
                    hiddenServiceMenuTimer.start()
                    hiddenServiceWatchdogTimer.start()
                }
            }
            else
                acceptKeyRelease = false
        }
        else if (event.key === keys.back)
        {
            if (event.isAutoRepeat === false && escapeSettingsModeTimer.running === false)
                escapeSettingsModeTimer.start()
        }
        else if (event.key === keys.onOff)
        {
            hideAll()
        }
    }

    Keys.onReleased: {
        if (acceptKeyRelease) {
            if (event.key === keys.back)
            {
                if (event.isAutoRepeat === false)
                {
                    if (escapeSettingsModeTimer.running)
                    {
                        escapeSettingsModeTimer.stop()
                        hide()
                    }
                }
                event.accepted = true
            }
            else if (event.key === keys.left && basePage === false)
            {
                hide()
                event.accepted = true
            }
            else if (event.key === keys.select || event.key === keys.right)
            {
                if (event.isAutoRepeat === false)
                {
                    if (hiddenServiceMenuTimer.running)
                    {
                        hiddenServiceMenuTimer.stop()
                        hiddenServiceWatchdogTimer.stop()
                    }
                    invokeCurrent(event.key)
                }
                event.accepted = true
            }
            else if (event.key === keys.up) {
                // Find next enabled item
                var oldIndex = settingsView.currentIndex
                for (var i = settingsView.currentIndex; i > 0; i--) {
                    settingsView.decrementCurrentIndex()
                    if (settingsView.currentItem !== null
                            && settingsView.currentItem.item.delegateItemEnabled
                            && settingsView.currentItem.item.delegateVisible)
                        break;
                }
                if (settingsView.currentItem !== null
                        && (settingsView.currentItem.item.delegateItemEnabled === false
                        || settingsView.currentItem.item.delegateVisible === false))
                    settingsView.currentIndex = oldIndex

                event.accepted = true
            }
            else if (event.key === keys.down) {
                // Find next enabled item
                var oldIndex2 = settingsView.currentIndex
                for (var j = settingsView.currentIndex; j < settingsView.count; j++) {
                    settingsView.incrementCurrentIndex()
                    if (settingsView.currentItem !== null
                            && settingsView.currentItem.item.delegateItemEnabled
                            && settingsView.currentItem.item.delegateVisible)
                        break;
                }
                if (settingsView.currentItem !== null
                        && (settingsView.currentItem.item.delegateItemEnabled === false
                        || settingsView.currentItem.item.delegateVisible === false))
                    settingsView.currentIndex = oldIndex2

                event.accepted = true
            }
            else if (event.key === keys.archive)
            {
                returnStateArchive = true
                hideAll()
            }
        }
        else
        {
            if (hiddenServiceMenuTimer.running)
            {
                if (hiddenServiceWatchdogTimer.running)
                    hiddenServiceWatchdogTimer.restart()
            }
        }
    }

    // The page itself, and its contents
    Rectangle {
        color: "black"
        width: grid.width
        height: grid.height

        // Header
        SettingsHeader {
            id: settingsHeader

            // "setting" is the text label propery. We'll show a specific header
            // if one is specified in the model (model.header) or we'll use the
            // name from the item which "pulled" us into this SettingsPage.
            setting: model !== undefined && model.header !== undefined && model.header !== "" ? qsTrId(model.header) + translation.update : name
            onClicked: hide()
        }

        // List
        ListView {
            id: settingsView

            anchors.top: settingsHeader.bottom
            width: parent.width
            anchors.bottom: settingsFooter.top
            clip: true  // Avoids animations outside list

            boundsBehavior: Flickable.StopAtBounds
            highlightMoveDuration: 5
            preferredHighlightBegin: grid.cellHeight + 2 * grid.verticalSpacing
            preferredHighlightEnd: height - grid.cellHeight - 2 * grid.verticalSpacing
            highlightRangeMode: ListView.ApplyRange

            highlight: Item {
                BorderImage {
                    source: "../images/Bg_FullScreen_Marker_Foc.png"
                    border.left: 5; border.top: 5; border.right: 5; border.bottom: 5
                    width: grid.width - 4 * grid.horizontalSpacing
                    height: grid.cellHeight - 2 * grid.verticalSpacing
                    z: 10
                    anchors.centerIn: parent
                }
            }

            delegate: Loader {
                id: delegateLoader
                sourceComponent: page.internalDelegate

                // Make ListView.isCurrentItem available for the delegate.
                property bool isCurrentItem: ListView.isCurrentItem

                // Bind our model to a property called model, for the delegate.
                Binding {
                    target: delegateLoader.item
                    property: "model"
                    value: model
                    when: delegateLoader.status === Loader.Ready
                }

                // Bind our settingsView to a property called view, for the
                // delegate.
                Binding {
                    target: delegateLoader.item
                    property: "view"
                    value: settingsView
                    when: delegateLoader.status === Loader.Ready
                }
            }
        }

        // Top scroll fader
        Image {
            id: topScrollCover
            visible: settingsView.visibleArea.yPosition > 0 ? true : false
            source: "../images/Bg_Options_ScrollCoverTop.png"
            anchors.top: settingsView.top
            x: grid.unit*2
        }

        // Bottom scroll fader
        Image {
            id: bottomScrollCover
            visible: (settingsView.contentHeight > settingsView.height) &&
                     (settingsView.visibleArea.yPosition < 1 - settingsView.visibleArea.heightRatio - 0.01) ? true : false
            source: "../images/Bg_Options_ScrollCoverBottom.png"
            anchors.bottom: settingsView.bottom
            x: grid.unit*2
        }

        // Scroll bar
        Item {
            id: scrollBar
            visible: settingsView.contentHeight > settingsView.height ? true : false
            anchors.right: parent.right
            anchors.top: settingsView.top
            anchors.bottom: settingsView.bottom
            width: 3

            // Background
            //
            // Suggestion: Changed this from an image to a rectangle. Same
            // colors, less waste of memory. Could it work?
            Rectangle {
//                source: "../images/Bg_Options_ScrollBarBg_Def.png"
                color: "#323232"
                anchors.fill: parent
            }

            // Slider
            //
            // Suggestion: Changed this from an image to a rectangle. Same
            // colors, less waste of memory. Could it work?
            Rectangle {
                color: "#0066cc"
                anchors.right: parent.right
                width: parent.width
//                source: "../images/Bg_Options_ScrollBarMarker_Def.png"
                height: settingsView.height * settingsView.visibleArea.heightRatio
                y: settingsView.visibleArea.yPosition * settingsView.height
            }
        }

        // Footer
        SettingsFooter {
            id: settingsFooter

            visible: model !== undefined
                     && model.footerEnabled !== undefined
                     && model.footerEnabled !== ""
                     && model.footerEnabled === true

            // Footer text either depends on the footer property of
            // SettingsModel (one footer for the entire view) or the itemFooter
            // property of SettingsItem (one footer per item).
            //
            // Item footer title have higher priority than the model/view footer
            // title.
            text: (settingsView !== undefined
                   && settingsView.currentItem !== null
                   && settingsView.currentItem.item !== undefined
                   && settingsView.currentItem.item.delegateFooterTitle !== undefined
                   && settingsView.currentItem.item.delegateFooterTitle !== ""
                   ? settingsView.currentItem.item.delegateFooterTitle
                   : model !== undefined
                     && model.footer !== undefined
                     && model.footer !== ""
                     ? qsTrId(model.footer)
                     : "") + translation.update
        }
    }

    // Loader used to load and focus next page
    Loader {
        id: pageLoader
        onLoaded: {
            //console.log("SettingsPage " + item.name + " loaded by " + name)
            item.previous = page
            page.next = item
            item.name = qsTrId(settingsView.currentItem.item.delegateLabel)
            item.basePage = false

            if (settingsView.currentItem.item.delegateSettingType === "multiEntry")
            {
                item.model1 = settingsView.currentItem.item.delegateNextModel1
                item.model2 = settingsView.currentItem.item.delegateNextModel2
                item.model3 = settingsView.currentItem.item.delegateNextModel3
                item.model4 = settingsView.currentItem.item.delegateNextModel4
                item.model5 = settingsView.currentItem.item.delegateNextModel5
                item.model6 = settingsView.currentItem.item.delegateNextModel6
                item.nextModels = settingsView.currentItem.item.delegateNextModels

                item.setCurrentIndex()
            }
            else
            {
                item.model = settingsView.currentItem.item.delegateNextModel
            }

            item.show()
        }
    }

    // Connection used to unload page after it has been hidden
    Connections {
        target: pageLoader.item
        onHideSignal: {
            //console.log(name + " unloading child: " + qsTrId(settingsView.currentItem.item.delegateLabel))
            pageLoader.source = ""
            if (model !== undefined && model.isNowActive !== undefined)
                model.isNowActive(true)
        }
    }

    // Loader used to display confirmation messages
    Loader {
        id: messageLoader
        onLoaded: {
            //console.log("messageLoader.onLoaded")
            item.title = qsTrId(settingsView.currentItem.item.delegateConfirmMsg) + translation.update
            item.text = qsTrId(settingsView.currentItem.item.delegateConfirmText) + translation.update
            item.cancelButtontext = qsTrId(settingsView.currentItem.item.delegateConfirmCancel) + translation.update
            item.okButtontext = qsTrId(settingsView.currentItem.item.delegateConfirmOk) + translation.update
            item.iconStyle = settingsView.currentItem.item.delegateConfirmIcon === "warning" ? item.iconEnum.warning :
                             settingsView.currentItem.item.delegateConfirmIcon === "info" ? item.iconEnum.info :
                             settingsView.currentItem.item.delegateConfirmIcon === "question" ? item.iconEnum.question :
                                                                                           item.iconEnum.error
            item.show()
            item.forceActiveFocus()
        }
    }

    // Connections used to handle response from message box and unload page after it has been hidden
    Connections {
        target: messageLoader.item
        onCancelSelected: {
            console.log("MessageBox unloading - cancel pressed")
            messageLoader.source = ""
            forceActiveFocus()
        }

        onOkSelected: {
            console.log("MessageBox unloading - ok pressed")
            messageLoader.source = ""
            forceActiveFocus()
            model.setProperty(settingsView.currentIndex,
                              settingsView.currentItem.item.delegateID,
                              settingsView.currentItem.item.delegateValue)
        }
    }

    // Slots needed to translate headers when chaning language (Complex but only solution?)
    function languageChanged()
    {
        if (settingsView.currentItem.item.delegateLabel !== "")
        {
            if (previous !== null)
                name = previous.getSettingName()
        }
        if (next !== null)
            next.languageChanged()
    }
    function getSettingName()
    {
        return qsTrId(settingsView.currentItem.item.delegateLabel)
    }
}
