import QtQuick 1.1
import GreenBox 1.0

Item {
    id: root
    property alias model: list.model
    property bool isRootLevelToolbar: false
    height: grid.cellHeight

    BorderImage {
        border { left: 10; top: 10; right: 10; bottom: 10 }
        source: "../images/Bg_MenuHorizontalMain_Bar_Def.png"

        id: toolbarBackground
        width: row.width
        height: row.height
        y: root.height - row.height
        x: root.isRootLevelToolbar ? (root.width - width) / 2 : calculateChildX()

        function calculateChildX()
        {
            var cellWidth = grid.cellWidth + grid.horizontalSpacing;
            var level1Width = cellWidth * menus.menuLevel1.length - grid.horizontalSpacing
            if (level1Width <= 0)
                return 0;
            var level1ToolBarXPos = (root.width - level1Width) / 2
            // start centered over the level1 toolbar item that is selected.
            var desiredX = cellWidth * (menus.selectedLevel1Menu + 0.5) - row.width / 2;

            // align to grid.
            desiredX -= desiredX % cellWidth;
            desiredX += level1ToolBarXPos; // add this after aligning to grid.

            if (desiredX < level1ToolBarXPos) { // left align with parent toolbar
                desiredX = level1ToolBarXPos;
            }
            else if (row.width > level1Width) {
                // doesn't happen right now ;)
                // basically we have to choose how to position the toolbar if/when we hit this.
                desiredX = grid.leftMargin
                console.debug("WARN child toolbar wider than parent, positioning hits undefined path")
            } else if (desiredX + row.width > level1Width + level1ToolBarXPos) { // right align
                desiredX = level1Width + level1ToolBarXPos - row.width
            }
            return desiredX
        }

        Image { // Open subtoolbar indicator
            id: arrow
            y: -grid.verticalSpacing
            x: (root.isRootLevelToolbar ? menus.selectedLevel1Menu : menus.selectedLevel2Menu) * (grid.cellWidth + grid.horizontalSpacing)
            visible:  row.hasPopupOpen || (root.isRootLevelToolbar && menus.selectedLevel1Menu !== -1)
            source: "../images/Bg_MenuHorizontalMain_Marker_Sel.png"
        }

        Flow {
            id: row
            property bool hasPopupOpen: false
            anchors.horizontalCenter: parent.horizontalCenter
            width: Math.min(root.width, Math.min(list.model.length, grid.horizontalCellCount) * (grid.cellWidth + grid.horizontalSpacing)) - grid.horizontalSpacing
            spacing: grid.horizontalSpacing

            Repeater {
                id: list
                Item {
                    width: grid.cellWidth
                    height: grid.cellHeight
                    id: menuItem
                    property bool isCurrent: menus.currentItem === modelData
                    focus: isCurrent

                    function execute() {
                        if (modelData.enabled) {
                            modelData.activate();
                            if (modelData.targetQmlFile !== "") {
                                // open a QML, and position it.
                                subItem.source = modelData.targetQmlFile
                                if (subItem.item != undefined) {
                                    row.hasPopupOpen = true
                                    subItem.item.model = modelData;
                                    subItem.item.x = (menuItem.width - subItem.width) / 2
                                    subItem.item.y = -subItem.item.height - 3
                                    subItem.item.forceActiveFocus()
                                }
                            }
                        }
                    }

                    Keys.onReleased: {
                        event.accepted = false;

                        if (event.key === keys.select) {
                            execute()
                            event.accepted = true
                        }
                    }

                    Image { // highlight
                        id: highLight
                        y: -grid.verticalSpacing
                        visible: menuItem.isCurrent
                        source: "../images/Bg_MenuHorizontalMain_Marker_Foc.png"
                    }

                    Text {
                        clip: true
                        anchors.fill: parent
                        text: modelData.title
                        color: modelData.enabled ? (parent.isCurrent ? colors.textFocused : colors.textNormal) : colors.textDisabled
                        horizontalAlignment: Text.AlignHCenter
                        font.pixelSize: fonts.smallSize
                        verticalAlignment: Text.AlignVCenter
                    }

                    Image {
                        id: icon
                        visible: modelData.title == ""
                        source: visible ? ("../images/" + modelData.icon +
                                (mouseArea.pressed && modelData.enabled ?
                                     "_Down.png" : (menuItem.isCurrent ?  "_Sel.png" : "_Def.png"))) : ""
                        anchors.centerIn: parent
                        opacity: modelData.enabled ? 1 : 0.3
                    }

                    Image {
                        id: checkedOverlay
                        anchors.top: parent.top
                        anchors.right: parent.right
                        visible: modelData.showCheckIndicator
                        source: "../images/Bg_MenuHorizontalMain_RadioButton" + (modelData.checked ? "On" : "Off") + "_Def.png"
                    }

                    RoundedRect { // Tooltip
                        opacity: (menuItem.isCurrent && !row.hasPopupOpen /* && timer.running */) ? 1 : 0
                        Text {
                            id: tooltip
                            text: qsTrId(modelData.tooltip)
                            x: grid.horizontalSpacing
                            y: 1
                            font.family: fonts.family
                            font.pixelSize: fonts.smallSize
                            color: colors.white
                        }
                        visible: false
                        width: tooltip.width + 6
                        height: tooltip.height + 4
                        y: -4 - height
                        Timer {
                            interval: 2000
                            running: menuItem.isCurrent
                            triggeredOnStart: true
                            id: timer
                            onTriggered: {
                                var wantedXPos = menuItem.width / 2 - parent.width / 2;
                                var posInBox = menuItem.mapToItem(root, wantedXPos, 0);
                                if (posInBox.x < grid.leftMargin + grid.unit) {
                                    wantedXPos += 0 - posInBox.x + grid.leftMargin + grid.unit
                                } else { // check if it falls out the right side too.
                                    posInBox = menuItem.mapToItem(root, wantedXPos + parent.width, 0);
                                    var right = grid.width - grid.leftMargin - grid.unit;
                                    if (posInBox.x > right) {
                                        var offset = posInBox.x - right;
                                        wantedXPos -= offset;
                                    }
                                }
                                parent.x = wantedXPos;
                                parent.visible = true
                            }
                        }
                    }

                    MouseArea {
                        id: mouseArea
                        anchors.fill: parent
                        onClicked: execute()
                    }
                    Loader {
                        id: subItem
                    }
                    Connections { target: menus
                        onCurrentItemChanged: {
                            // make sure we auto-close the popup if the user clicked another toolbar item
                            if (menus.currentItem != modelData)
                                subItem.source = ""
                        }
                    }
                    Connections { target: subItem.item
                        onVisibleChanged: { // allow the popup subItem to close itself.
                            subItem.source = ""
                            row.hasPopupOpen = false
                            menuItem.forceActiveFocus()
                        }
                    }
                    // debug
                    // Text { text: "newToolbar has focus " + parent.activeFocus; y: -30; color:"yellow"; visible: isCurrent }
                }
                onItemAdded: {
                    if (item.isCurrent)
                        item.forceActiveFocus()
                }
                onItemRemoved: {
                    root.focus = false
                    row.hasPopupOpen = false
                }
            }
        }
    }
    Keys.onDownPressed: menus.navigateDown()
    Keys.onLeftPressed: menus.navigateLeft()
    Keys.onRightPressed: menus.navigateRight()
    Keys.onUpPressed: menus.navigateUp()
    Keys.onPressed: {
        event.accepted = false;
        if (event.key === keys.onOff)
        {
            greenbox.system.key_down(GreenBox.UI_KEY_ONOFF,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
    }
    Keys.onReleased: {
        event.accepted = false;
        if (event.key === keys.back) {
            menus.navigateDown()
            event.accepted = true
        }
        else if (event.key === keys.archive || event.key === keys.trigger) {
            menus.closeMenu()
            event.accepted = true
        }
        else if (event.key === keys.onOff)
        {
            menus.closeMenu()
            greenbox.system.key_up(GreenBox.UI_KEY_ONOFF,event.isAutoRepeat === true?1:0)
            event.accepted = true
        }
    }
}
